# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-04 18:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_cascade', '0007_add_proxy_models'),
    ]

    operations = [
        migrations.CreateModel(
            name='SortableInlineCascadeElement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('glossary', jsonfield.fields.JSONField(blank=True, default={})),
                ('order', models.PositiveIntegerField(db_index=True, verbose_name='Sort by')),
                ('cascade_element', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sortinline_elements', to='cmsplugin_cascade.CascadeElement')),
            ],
            options={
                'ordering': ('order',),
                'db_table': 'cmsplugin_cascade_sortinline',
            },
        ),
    ]
