form input:disabled, form textarea:disabled, form select:disabled { color: #888; border-style: dashed; }
form .form-row.field-link_type { min-height: 30px; padding-left: 0; padding-right: 0; }
form .form-row.field-link_type  label { font-size: 120%; }
form .form-row.field-link_type  label.inline { width: 0; }
form .form-row.field-link_type input { margin-top: 0; }
form .form-row.field-link_type .help { margin-left: 2em; padding-left: 0; }
form .form-row.field-link_type .field-link_type .help { margin-left: 160px; }
form .form-row.field-cms_page { height: 75px; float: left; }
form .form-row.field-section { height: 75px; }
form .form-row.field-section > div { margin-top: 4px; }
form .form-row.field-section::after { content: "."; display: block; clear: both; visibility: hidden; height: 0; }
form .field-download_file { width: 100%; }
form  label.inline { text-align: right; }
form .field-glossary { padding: 0 10px; }

form .container-thumbnail img { display: inline-block; }
form .container-thumbnail .label { display: inline; }

form .form-row.field-save_settings_as input[type="checkbox"] {
  margin-right: 2em;
}


/* form .container-thumbnail i[class^="icon"] { font-size: 70px; }*/

form label[for=id_cms_page], form label[for=id_section] { width: 0; }
#id_section + .help { margin-left: 0; }
#id_ext_url, #id_mail_to { min-width: 350px !important;}
.field-box > .select2-container, .fieldBox > .select2-container { min-width: 350px; }
.field-box.field-link_type, .fieldBox.field-link_type { float: none; margin-bottom: 15px; }

#id_condition, #id_link_content { width: calc(100% - 20px); }

form .form-row .select2-selection { height: 36px; }
form .form-row .django-select2 { width: calc(100% - 200px) !important; }